@@ -20,6 +20,8 @@ import contextlib
 import functools
 import importlib
 import operator
+import threading
+import traceback
 import uuid
 
 from .util import _noop_callback
@@ -33,16 +35,17 @@ class Graph:
         self.operations = {}
         self.dependencies = []
 
-    default_graph = None
+    _globals = threading.local()
 
     def __enter__(self):
-        assert self.default_graph is None, "cannot have more than one default graph"
-        Graph.default_graph = self
+        assert getattr(self._globals, 'default_graph', None) is  None, \
+            "cannot have more than one default graph"
+        Graph._globals.default_graph = self
         return self
 
     def __exit__(self, *args):
-        assert self.default_graph is self
-        Graph.default_graph = None
+        assert self._globals.default_graph is self
+        Graph._globals.default_graph = None
 
     def normalize_operation(self, operation):  # pylint:disable=W0621
         """
@@ -71,10 +74,9 @@ class Graph:
             if operation.graph is not self:
                 raise RuntimeError(f"operation '{operation}' does not belong to this graph")
             return operation
-        elif isinstance(operation, str):
+        if isinstance(operation, str):
             return self.operations[operation]
-        else:
-            raise ValueError(f"'{operation}' is not an `Operation` instance or operation name")
+        raise ValueError(f"'{operation}' is not an `Operation` instance or operation name")
 
     def normalize_context(self, context, **kwargs):
         """
@@ -159,7 +161,7 @@ class Graph:
 
         fetches = [self.normalize_operation(operation) for operation in fetches]
         context = self.normalize_context(context, **kwargs)
-        values = [fetch.evaluate(context, callback=callback) for fetch in fetches]
+        values = [fetch.evaluate_operation(fetch, context, callback=callback) for fetch in fetches]
         return values[0] if single else tuple(values)
 
     __call__ = apply
@@ -183,13 +185,19 @@ class Graph:
         ValueError
             If no `Graph` instance can be obtained.
         """
-        graph = graph or Graph.default_graph
+        graph = graph or Graph._globals.default_graph
         if not graph:
             raise ValueError("`graph` must be given explicitly or a default graph must be set")
         return graph
 
 
-class Operation:  # pylint:disable=too-few-public-methods
+class EvaluationError(RuntimeError):
+    """
+    Failed to evaluate an operation.
+    """
+
+
+class Operation:  # pylint:disable=too-few-public-methods,too-many-instance-attributes
     """
     Base class for operations.
 
@@ -221,6 +229,8 @@ class Operation:  # pylint:disable=too-few-public-methods
         # Get a list of all dependencies relevant to this operation
         self.dependencies = [] if dependencies is None else dependencies
         self.dependencies.extend(self.graph.dependencies)
+        # Get the stack context so we can report where the operation was defined
+        self._stack = traceback.extract_stack()
 
     def __getstate__(self):
         return self.__dict__
@@ -322,18 +332,36 @@ class Operation:  # pylint:disable=too-few-public-methods
         """
         Evaluate an operation or constant given a context.
         """
-        if isinstance(operation, Operation):
-            return operation.evaluate(context, **kwargs)
-        partial = functools.partial(cls.evaluate_operation, context=context, **kwargs)
-        if isinstance(operation, tuple):
-            return tuple(partial(element) for element in operation)
-        if isinstance(operation, list):
-            return [partial(element) for element in operation]
-        if isinstance(operation, dict):
-            return {partial(key): partial(value) for key, value in operation.items()}
-        if isinstance(operation, slice):
-            return slice(*[partial(getattr(operation, attr)) for attr in ['start', 'stop', 'step']])
-        return operation
+        try:
+            if isinstance(operation, Operation):
+                return operation.evaluate(context, **kwargs)
+            partial = functools.partial(cls.evaluate_operation, context=context, **kwargs)
+            if isinstance(operation, tuple):
+                return tuple(partial(element) for element in operation)
+            if isinstance(operation, list):
+                return [partial(element) for element in operation]
+            if isinstance(operation, dict):
+                return {partial(key): partial(value) for key, value in operation.items()}
+            if isinstance(operation, slice):
+                return slice(*[partial(getattr(operation, attr))
+                               for attr in ['start', 'stop', 'step']])
+            return operation
+        except Exception as ex:  # pragma: no cover
+            stack = []
+            interactive = False
+            for frame in reversed(operation._stack):  # pylint: disable=protected-access
+                # Do not capture any internal stack traces
+                if 'pythonflow' in frame.filename:
+                    continue
+                # Stop tracing at the last interactive cell
+                if interactive and not frame.filename.startswith('<'):
+                    break  # pragma: no cover
+                interactive = frame.filename.startswith('<')
+                stack.append(frame)
+
+            stack = "".join(traceback.format_list(reversed(stack)))
+            message = "Failed to evaluate operation `%s` defined at:\n\n%s" % (operation, stack)
+            raise ex from EvaluationError(message)
 
     def __bool__(self):
         return True
